﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WP7Push_Notifier.Tools;
using WP7Push_Notifier.Service;
using System.Threading;

namespace WP7Push_Notifier
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private NotificationSender notificationSender = new NotificationSender();

        public MainWindow()
        {
            InitializeComponent();
            SubscribersRegistrationService.Subscribed += new EventHandler<SubscribersRegistrationService.SubscriptionEventArgs>(SubscribersRegistrationService_Subscribed);
        }

        private void btnSendRaw_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            List<Uri> subscribers = SubscribersRegistrationService.GetSubscribers();
            byte[] payload = PrepareRAWPayload(this.tbxRawMessage.Text);
            notificationSender.SendRawNotification(subscribers, payload);
        }

        private void btnSendToast_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            List<Uri> subscribers = SubscribersRegistrationService.GetSubscribers();
            notificationSender.SendToastNotification(subscribers, this.tbxToastMessage1.Text, this.tbxToastMessage2.Text);
        }

        private void btnSendTile_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            List<Uri> subscribers = SubscribersRegistrationService.GetSubscribers();
            string imageUrl = "";
            if (this.cbxRemoteUrl.IsChecked.Value)
            {
                imageUrl = String.Format("http://maciejgrabek.com/articles-upload/WP7/Push/Images/{0}.png", ((ComboBoxItem)this.cbxTileBackground.SelectedItem).Content);
            }
            else
            {
                imageUrl = String.Format("/Images/{0}.png", ((ComboBoxItem)this.cbxTileBackground.SelectedItem).Content);
            }

            notificationSender.SendTileNotification(subscribers, imageUrl, int.Parse(this.tbxTileCount.Text), this.tbxTileTitle.Text);
        }

        private void SubscribersRegistrationService_Subscribed(object sender, SubscribersRegistrationService.SubscriptionEventArgs e)
        {
            List<Uri> subscribers = SubscribersRegistrationService.GetSubscribers();
            string subsStr = "Registered subscribers\n";
            foreach (var item in subscribers)
            {
                subsStr += item.AbsoluteUri + "\n";
            }

            this.tbxRegisteredItems.Text = subsStr;
        }

        private byte[] PrepareRAWPayload(string data)
        {
            string rawDataSkeleton = "<?xml version=\"1.0\" encoding=\"utf-8\"?>" + 
                "<MyRawMessage>" +
                    "<Content>{0}</Content>" +
                "</MyRawMessage>";
            return Encoding.UTF8.GetBytes(String.Format(rawDataSkeleton, data));
        }
    }
}
