﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Notification;
using System.Collections.ObjectModel;
using System.IO;
using System.Xml.Linq;

namespace WP7Push_Subscriber
{
    public partial class MainPage : PhoneApplicationPage
    {
        private HttpNotificationChannel httpChannel;
        const string channelName = "WP7Push_ChannelTest1";

        // Constructor
        public MainPage()
        {
            InitializeComponent();
            ConnectToChannel();
            #region UI values
            this.tbxMessageFormat.Text =
                "<?xml version=\"1.0\" encoding=\"utf-8\"?>" + Environment.NewLine +
                "<MyRawMessage>" + Environment.NewLine +
                    "<Content>{0}</Content>" + Environment.NewLine + 
                "</MyRawMessage>";
            #endregion
        }

        private void ConnectToChannel()
        {
            try
            {
                //First, try to pick up existing channel
                httpChannel = HttpNotificationChannel.Find(channelName);

                if (null != httpChannel)
                {
                    Trace("Channel Exists");
                    SubscribeToChannelEvents();
                    SubscribeToService();

                    Trace("Subscribe the channel to Toast and Tile notifications");
                    SubscribeToNotifications();

                    Dispatcher.BeginInvoke(() => UpdateStatus("Channel recovered"));
                }
                else
                {
                    Trace("Trying to create a new channel...");
                    //Create the channel
                    httpChannel = new HttpNotificationChannel(channelName);
                    Trace("New Push Notification channel created successfully");

                    SubscribeToChannelEvents();

                    Trace("Trying to open the channel");
                    httpChannel.Open();
                    Dispatcher.BeginInvoke(() => UpdateStatus("Channel open requested"));
                }

            }
            catch (Exception ex)
            {
                Dispatcher.BeginInvoke(() => UpdateStatus("Channel error: " + ex.Message));
            }
        }

        private void SubscribeToChannelEvents()
        {
            //Register to UriUpdated event - occurs when channel successfully opens
            httpChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(httpChannel_ChannelUriUpdated);

            //general error handling for push channel
            httpChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(httpChannel_ExceptionOccurred);

            //Subscribed to Raw Notification
            httpChannel.HttpNotificationReceived += new EventHandler<HttpNotificationEventArgs>(httpChannel_HttpNotificationReceived);

            //subscrive to toast notification when running app    
            httpChannel.ShellToastNotificationReceived += new EventHandler<NotificationEventArgs>(httpChannel_ShellToastNotificationReceived);
        }

        private void SubscribeToService()
        {
            string baseUri = "http://localhost:2010/SubscribersRegistrationService/Register?uri={0}";
            string theUri = String.Format(baseUri, httpChannel.ChannelUri.ToString());
            WebClient client = new WebClient();
            client.DownloadStringCompleted += (s, e) =>
            {
                if (null == e.Error)
                    Dispatcher.BeginInvoke(() => UpdateStatus("Registration succeeded"));
                else
                    Dispatcher.BeginInvoke(() => UpdateStatus("Registration failed: " + e.Error.Message));
            };
            client.DownloadStringAsync(new Uri(theUri));
        }

        private void SubscribeToNotifications()
        {
            // Bind to Toast Notification 
            try
            {
                if (httpChannel.IsShellToastBound == true)
                {
                    Trace("Already bounded (register) to Toast notification");
                }
                else
                {
                    Trace("Registering to Toast Notifications");
                    httpChannel.BindToShellToast();
                }
            }
            catch (Exception ex) { }

            // Bind to Tile Notification 
            try
            {
                if (httpChannel.IsShellTileBound == true)
                {
                    Trace("Already bounded (register) to Tile Notifications");
                }
                else
                {
                    Trace("Registering to Tile Notifications");

                    //optional:
                    //register allowed background images url
                    //they have to be visible for microsoft push notification service!
                    Collection<Uri> uris = new Collection<Uri>();
                    uris.Add(new Uri("http://maciejgrabek.com/articles-upload/WP7/Push/Images/blue.png"));
                    uris.Add(new Uri("http://maciejgrabek.com/articles-upload/WP7/Push/Images/green.png"));
                    uris.Add(new Uri("http://maciejgrabek.com/articles-upload/WP7/Push/Images/red.png"));

                    httpChannel.BindToShellTile(uris);
                }
            }
            catch (Exception ex) { }
        }

        private void httpChannel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            Trace("Channel opened. Got Uri:\n" + httpChannel.ChannelUri.ToString());
            
            Trace("Subscribing to channel events");
            SubscribeToService();
            SubscribeToNotifications();
            Dispatcher.BeginInvoke(() => UpdateStatus("Channel created successfully"));
        }

        private void httpChannel_ExceptionOccurred(object sender, NotificationChannelErrorEventArgs e)
        {
            Dispatcher.BeginInvoke(() => UpdateStatus(e.ErrorType + " occurred: " + e.Message));
        }

        #region RAW
        private void httpChannel_HttpNotificationReceived(object sender, HttpNotificationEventArgs e)
        {
            Trace("RAW notification arrived:");
            string result = ParseRawMessage(e.Notification.Body);
            Dispatcher.BeginInvoke(() => UpdateStatus("RAW message: " + Environment.NewLine +  result));
            Trace(result);
        }

        private string ParseRawMessage(System.IO.Stream stream)
        {
            //read your data
            XDocument document;
            using (var reader = new StreamReader(stream))
            {
                string payload = reader.ReadToEnd().Replace('\0', ' ');
                document = XDocument.Parse(payload);
            }

            return (from c in document.Descendants("MyRawMessage") select c.Element("Content").Value).FirstOrDefault();
        }
        #endregion

        #region Toast
        void httpChannel_ShellToastNotificationReceived(object sender, NotificationEventArgs e)
        {
            Trace("Toast notification arrived:");
            string msg = "";
            foreach (var key in e.Collection.Keys)
            {
                msg += e.Collection[key] + Environment.NewLine;
            }

            Trace(msg);
            Dispatcher.BeginInvoke(() => UpdateStatus("Toast message: " + Environment.NewLine + msg));
        }
        #endregion

        private void UpdateStatus(string message)
        {
            tbxStatus.Text = message;
        }

        private void Trace(string message)
        {
#if DEBUG
            System.Diagnostics.Debug.WriteLine(message);
#endif
        }
    }
}